@@ -1,15 +1,30 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* e-cell-tree.c - Tree cell renderer
- * Copyright (C) 2000 Ximian, Inc.
+/* 
+ * e-cell-tree.c - Tree cell object.
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Author: Chris Toshok <toshok@ximian.com>
+ * Authors:
+ *   Chris Toshok <toshok@ximian.com>
  *
  * A majority of code taken from:
  *
  * the ECellText renderer.
+ * Copyright 1998, The Free Software Foundation
+ * Copyright 1999, 2000, Ximian, Inc.
  *
- * Copyright (C) 1998 The Free Software Foundation
- * Copyright (C) 1999, 2000 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -79,6 +94,9 @@ visible_depth_of_node (ETableModel *model, int row)
 		- (e_tree_table_adapter_root_node_is_visible (adapter) ? 0 : 1));
 }
 
+/* If this is changed to not include the width of the expansion pixmap
+   if the path is not expandable, then max_width needs to change as
+   well. */
 static gint
 offset_of_node (ETableModel *table_model, int row)
 {
@@ -410,27 +428,29 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 	int number_of_rows;
 	int max_width = 0;
 	int width = 0;
-	int subcell_max_width;
+	int subcell_max_width = 0;
+	gboolean per_row = e_cell_max_width_by_row_implemented (tree_view->subcell_view);
 
 	number_of_rows = e_table_model_row_count (ecell_view->e_table_model);
 
-	subcell_max_width = e_cell_max_width (tree_view->subcell_view, model_col, view_col);
+	if (!per_row)
+		subcell_max_width = e_cell_max_width (tree_view->subcell_view, model_col, view_col);
 	
 	for (row = 0; row < number_of_rows; row++) {
 		ETreeModel *tree_model = e_cell_tree_get_tree_model(ecell_view->e_table_model, row);
-		ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
 		ETreePath node;
 		GdkPixbuf *node_image;
 		int node_image_width = 0, node_image_height = 0;
 		
 		int offset, subcell_offset;
+#if 0
 		gboolean expanded, expandable;
+		ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
+#endif
 		
 		node = e_cell_tree_get_node (ecell_view->e_table_model, row);
 		
 		offset = offset_of_node (ecell_view->e_table_model, row);
-		expandable = e_tree_model_node_is_expandable (tree_model, node);
-		expanded = e_tree_table_adapter_node_is_expanded (tree_table_adapter, node);
 		subcell_offset = offset;
 
 		node_image = e_tree_model_icon_at (tree_model, node);
@@ -440,7 +460,20 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 			node_image_height = gdk_pixbuf_get_height (node_image);
 		}
 
-		width = subcell_max_width + subcell_offset + node_image_width;
+		width = subcell_offset + node_image_width;
+
+		if (per_row)
+			width += e_cell_max_width_by_row (tree_view->subcell_view, model_col, view_col, row);
+		else
+			width += subcell_max_width;
+
+#if 0
+		expandable = e_tree_model_node_is_expandable (tree_model, node);
+		expanded = e_tree_table_adapter_node_is_expanded (tree_table_adapter, node);
+
+		/* This is unnecessary since this is already handled
+                   by the offset_of_node function.  If that changes,
+                   this will have to change too. */
 
 		if (expandable) {
 			GdkPixbuf *image;
@@ -451,6 +484,7 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 
 			width += gdk_pixbuf_get_width(image);
 		}
+#endif
 
 		max_width = MAX (max_width, width);
 	}
@@ -664,7 +698,7 @@ e_cell_tree_class_init (GtkObjectClass *object_class)
 	parent_class = gtk_type_class (PARENT_TYPE);
 }
 
-E_MAKE_TYPE(e_cell_tree, "ECellTree", ECellTree, e_cell_tree_class_init, NULL, PARENT_TYPE);
+E_MAKE_TYPE(e_cell_tree, "ECellTree", ECellTree, e_cell_tree_class_init, NULL, PARENT_TYPE)
 
 /**
  * e_cell_tree_construct:
