@@ -1,16 +1,30 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-subset.c: Implements a table that contains a subset of another table.
+/* 
+ * e-table-subset-variable.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 1999 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <stdlib.h>
-#include <gtk/gtksignal.h>
 #include <string.h>
+#include <gtk/gtksignal.h>
 #include "gal/util/e-util.h"
 #include "e-table-subset-variable.h"
 
@@ -32,8 +46,8 @@ etssv_add       (ETableSubsetVariable *etssv,
 	e_table_model_pre_change(etm);
 
 	if (etss->n_map + 1 > etssv->n_vals_allocated){
-		etss->map_table = g_realloc (etss->map_table, (etssv->n_vals_allocated + INCREMENT_AMOUNT) * sizeof(int));
 		etssv->n_vals_allocated += INCREMENT_AMOUNT;
+		etss->map_table = g_realloc (etss->map_table, etssv->n_vals_allocated * sizeof(int));
 	}
 
 	etss->map_table[etss->n_map++] = row;
@@ -41,6 +55,27 @@ etssv_add       (ETableSubsetVariable *etssv,
 	e_table_model_row_inserted (etm, etss->n_map - 1);
 }
 
+static void
+etssv_add_array (ETableSubsetVariable *etssv,
+		 const gint           *array,
+		 gint                  count)
+{
+	ETableModel *etm = E_TABLE_MODEL(etssv);
+	ETableSubset *etss = E_TABLE_SUBSET(etssv);
+	int i;
+
+	e_table_model_pre_change(etm);
+	
+	if (etss->n_map + count > etssv->n_vals_allocated){
+		etssv->n_vals_allocated += MAX(INCREMENT_AMOUNT, count);
+		etss->map_table = g_realloc (etss->map_table, etssv->n_vals_allocated * sizeof(int));
+	}
+	for (i = 0; i < count; i++)
+		etss->map_table[etss->n_map++] = array[i];
+
+	e_table_model_changed (etm);
+}
+
 static void
 etssv_add_all   (ETableSubsetVariable *etssv)
 {
@@ -90,11 +125,12 @@ etssv_class_init (GtkObjectClass *object_class)
 	etssv_parent_class = gtk_type_class (PARENT_TYPE);
 
 	klass->add     = etssv_add;
+	klass->add_array = etssv_add_array;
 	klass->add_all = etssv_add_all;
 	klass->remove  = etssv_remove;
 }
 
-E_MAKE_TYPE(e_table_subset_variable, "ETableSubsetVariable", ETableSubsetVariable, etssv_class_init, NULL, PARENT_TYPE);
+E_MAKE_TYPE(e_table_subset_variable, "ETableSubsetVariable", ETableSubsetVariable, etssv_class_init, NULL, PARENT_TYPE)
 
 ETableModel *
 e_table_subset_variable_construct (ETableSubsetVariable *etssv,
@@ -131,6 +167,18 @@ e_table_subset_variable_add       (ETableSubsetVariable *etssv,
 		ETSSV_CLASS (etssv)->add (etssv, row);
 }
 
+void
+e_table_subset_variable_add_array   (ETableSubsetVariable *etssv,
+				     const gint           *array,
+				     gint                  count)
+{
+	g_return_if_fail (etssv != NULL);
+	g_return_if_fail (E_IS_TABLE_SUBSET_VARIABLE(etssv));
+
+	if (ETSSV_CLASS(etssv)->add_array)
+		ETSSV_CLASS (etssv)->add_array (etssv, array, count);
+}
+
 void
 e_table_subset_variable_add_all   (ETableSubsetVariable *etssv)
 {
@@ -154,6 +202,21 @@ e_table_subset_variable_remove    (ETableSubsetVariable *etssv,
 		return FALSE;
 }
 
+void
+e_table_subset_variable_clear (ETableSubsetVariable *etssv)
+{
+	ETableModel *etm = E_TABLE_MODEL(etssv);
+	ETableSubset *etss = E_TABLE_SUBSET(etssv);
+
+	e_table_model_pre_change (etm);
+	etss->n_map = 0;
+	g_free (etss->map_table);
+	etss->map_table = g_new (unsigned int, 1);
+	etssv->n_vals_allocated = 1;
+	
+	e_table_model_changed (etm);
+}
+
 void
 e_table_subset_variable_increment (ETableSubsetVariable *etssv,
 				   gint                  position,
