@@ -1,12 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-subset.c: Implements a table that contains a subset of another table.
+/* 
+ * e-table-subset.c - Implements a table that contains a subset of another table.
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza <miguel@ximian.com>
  *
- * (C) 1999 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <stdlib.h>
 #include <gtk/gtksignal.h>
@@ -14,9 +29,12 @@
 #include "e-table-subset.h"
 
 static void etss_proxy_model_pre_change_real (ETableSubset *etss, ETableModel *etm);
+static void etss_proxy_model_no_change_real (ETableSubset *etss, ETableModel *etm);
 static void etss_proxy_model_changed_real (ETableSubset *etss, ETableModel *etm);
 static void etss_proxy_model_row_changed_real (ETableSubset *etss, ETableModel *etm, int row);
 static void etss_proxy_model_cell_changed_real (ETableSubset *etss, ETableModel *etm, int col, int row);
+static void etss_proxy_model_rows_inserted_real (ETableSubset *etss, ETableModel *etm, int row, int count);
+static void etss_proxy_model_rows_deleted_real (ETableSubset *etss, ETableModel *etm, int row, int count);
 
 #define PARENT_TYPE E_TABLE_MODEL_TYPE
 #define d(x)
@@ -25,16 +43,21 @@ static ETableModelClass *etss_parent_class;
 
 #define ETSS_CLASS(object) (E_TABLE_SUBSET_CLASS(GTK_OBJECT(object)->klass))
 
+#define VALID_ROW(etss, row) (row >= -1 && row < etss->n_map)
+#define MAP_ROW(etss, row) (row == -1 ? -1 : etss->map_table[row])
+
 static gint
 etss_get_view_row (ETableSubset *etss, int row)
 {
-	int limit;
 	const int n = etss->n_map;
 	const int * const map_table = etss->map_table;
 	int i;
 
-	limit = MIN(n, etss->last_access + 10);
-	for (i = etss->last_access; i < limit; i++) {
+	int end = MIN(etss->n_map, etss->last_access + 10);
+	int start = MAX(0, etss->last_access - 10);
+	int initial = MAX (MIN (etss->last_access, end), start);
+
+	for (i = initial; i < end; i++) {
 		if (map_table [i] == row){
 			d(g_print("a) Found %d from %d\n", i, etss->last_access));
 			etss->last_access = i;
@@ -42,10 +65,8 @@ etss_get_view_row (ETableSubset *etss, int row)
 		}
 	}
 
-	limit = MAX(0, etss->last_access - 10);
-	for (i = etss->last_access - 1; i >= limit; i--) {
+	for (i = initial - 1; i >= start; i--) {
 		if (map_table [i] == row){
-			e_table_model_row_changed (E_TABLE_MODEL (etss), i);
 			d(g_print("b) Found %d from %d\n", i, etss->last_access));
 			etss->last_access = i;
 			return i;
@@ -54,7 +75,6 @@ etss_get_view_row (ETableSubset *etss, int row)
 
 	for (i = 0; i < n; i++){
 		if (map_table [i] == row){
-			e_table_model_row_changed (E_TABLE_MODEL (etss), i);
 			d(g_print("c) Found %d from %d\n", i, etss->last_access));
 			etss->last_access = i;
 			return i;
@@ -71,6 +91,8 @@ etss_destroy (GtkObject *object)
 	if (etss->source) {
 		gtk_signal_disconnect (GTK_OBJECT (etss->source),
 				       etss->table_model_pre_change_id);
+		gtk_signal_disconnect (GTK_OBJECT (etss->source),
+				       etss->table_model_no_change_id);
 		gtk_signal_disconnect (GTK_OBJECT (etss->source),
 				       etss->table_model_changed_id);
 		gtk_signal_disconnect (GTK_OBJECT (etss->source),
@@ -118,9 +140,11 @@ etss_value_at (ETableModel *etm, int col, int row)
 {
 	ETableSubset *etss = (ETableSubset *)etm;
 
+	g_return_val_if_fail (VALID_ROW (etss, row), NULL);
+
 	etss->last_access = row;
 	d(g_print("g) Setting last_access to %d\n", row));
-	return e_table_model_value_at (etss->source, col, etss->map_table [row]);
+	return e_table_model_value_at (etss->source, col, MAP_ROW(etss, row));
 }
 
 static void
@@ -128,9 +152,11 @@ etss_set_value_at (ETableModel *etm, int col, int row, const void *val)
 {
 	ETableSubset *etss = (ETableSubset *)etm;
 
+	g_return_if_fail (VALID_ROW (etss, row));
+
 	etss->last_access = row;
 	d(g_print("h) Setting last_access to %d\n", row));
-	e_table_model_set_value_at (etss->source, col, etss->map_table [row], val);
+	e_table_model_set_value_at (etss->source, col, MAP_ROW(etss, row), val);
 }
 
 static gboolean
@@ -138,15 +164,15 @@ etss_is_cell_editable (ETableModel *etm, int col, int row)
 {
 	ETableSubset *etss = (ETableSubset *)etm;
 
-	return e_table_model_is_cell_editable (etss->source, col, etss->map_table [row]);
+	g_return_val_if_fail (VALID_ROW (etss, row), FALSE);
+
+	return e_table_model_is_cell_editable (etss->source, col, MAP_ROW(etss, row));
 }
 
 static gboolean
 etss_has_save_id (ETableModel *etm)
 {
-	ETableSubset *etss = (ETableSubset *)etm;
-
-	return e_table_model_has_save_id (etss->source);
+	return TRUE;
 }
 
 static char *
@@ -154,7 +180,12 @@ etss_get_save_id (ETableModel *etm, int row)
 {
 	ETableSubset *etss = (ETableSubset *)etm;
 
-	return e_table_model_get_save_id (etss->source, etss->map_table [row]);
+	g_return_val_if_fail (VALID_ROW (etss, row), NULL);
+
+	if (e_table_model_has_save_id (etss->source))
+		return e_table_model_get_save_id (etss->source, MAP_ROW(etss, row));
+	else
+		return g_strdup_printf ("%d", MAP_ROW(etss, row));
 }
 
 static void
@@ -207,36 +238,37 @@ etss_value_to_string (ETableModel *etm, int col, const void *value)
 static void
 etss_class_init (GtkObjectClass *object_class)
 {
-	ETableSubsetClass *klass        = (ETableSubsetClass *) object_class;
-	ETableModelClass *table_class   = (ETableModelClass *) object_class;
+	ETableSubsetClass *klass         = (ETableSubsetClass *) object_class;
+	ETableModelClass *table_class    = (ETableModelClass *) object_class;
 
-	etss_parent_class               = gtk_type_class (PARENT_TYPE);
+	etss_parent_class                = gtk_type_class (PARENT_TYPE);
 	
-	object_class->destroy           = etss_destroy;
+	object_class->destroy            = etss_destroy;
 
-	table_class->column_count       = etss_column_count;
-	table_class->row_count          = etss_row_count;
-	table_class->append_row         = etss_append_row;
+	table_class->column_count        = etss_column_count;
+	table_class->row_count           = etss_row_count;
+	table_class->append_row          = etss_append_row;
 
-	table_class->value_at           = etss_value_at;
-	table_class->set_value_at       = etss_set_value_at;
-	table_class->is_cell_editable   = etss_is_cell_editable;
+	table_class->value_at            = etss_value_at;
+	table_class->set_value_at        = etss_set_value_at;
+	table_class->is_cell_editable    = etss_is_cell_editable;
 
-	table_class->has_save_id        = etss_has_save_id;
-	table_class->get_save_id        = etss_get_save_id;
+	table_class->has_save_id         = etss_has_save_id;
+	table_class->get_save_id         = etss_get_save_id;
 
-	table_class->duplicate_value    = etss_duplicate_value;
-	table_class->free_value         = etss_free_value;
-	table_class->initialize_value   = etss_initialize_value;
-	table_class->value_is_empty     = etss_value_is_empty;
-	table_class->value_to_string    = etss_value_to_string;
+	table_class->duplicate_value     = etss_duplicate_value;
+	table_class->free_value          = etss_free_value;
+	table_class->initialize_value    = etss_initialize_value;
+	table_class->value_is_empty      = etss_value_is_empty;
+	table_class->value_to_string     = etss_value_to_string;
 
-	klass->proxy_model_pre_change   = etss_proxy_model_pre_change_real;
-	klass->proxy_model_changed      = etss_proxy_model_changed_real;
-	klass->proxy_model_row_changed  = etss_proxy_model_row_changed_real;
-	klass->proxy_model_cell_changed = etss_proxy_model_cell_changed_real;
-	klass->proxy_model_rows_inserted = NULL;
-	klass->proxy_model_rows_deleted  = NULL;
+	klass->proxy_model_pre_change    = etss_proxy_model_pre_change_real;
+	klass->proxy_model_no_change     = etss_proxy_model_no_change_real;
+	klass->proxy_model_changed       = etss_proxy_model_changed_real;
+	klass->proxy_model_row_changed   = etss_proxy_model_row_changed_real;
+	klass->proxy_model_cell_changed  = etss_proxy_model_cell_changed_real;
+	klass->proxy_model_rows_inserted = etss_proxy_model_rows_inserted_real;
+	klass->proxy_model_rows_deleted  = etss_proxy_model_rows_deleted_real;
 }
 
 static void
@@ -245,7 +277,7 @@ etss_init (ETableSubset *etss)
 	etss->last_access = 0;
 }
 
-E_MAKE_TYPE(e_table_subset, "ETableSubset", ETableSubset, etss_class_init, etss_init, PARENT_TYPE);
+E_MAKE_TYPE(e_table_subset, "ETableSubset", ETableSubset, etss_class_init, etss_init, PARENT_TYPE)
 
 static void
 etss_proxy_model_pre_change_real (ETableSubset *etss, ETableModel *etm)
@@ -253,6 +285,12 @@ etss_proxy_model_pre_change_real (ETableSubset *etss, ETableModel *etm)
 	e_table_model_pre_change (E_TABLE_MODEL (etss));
 }
 
+static void
+etss_proxy_model_no_change_real (ETableSubset *etss, ETableModel *etm)
+{
+	e_table_model_no_change (E_TABLE_MODEL (etss));
+}
+
 static void
 etss_proxy_model_changed_real (ETableSubset *etss, ETableModel *etm)
 {
@@ -265,6 +303,8 @@ etss_proxy_model_row_changed_real (ETableSubset *etss, ETableModel *etm, int row
 	int view_row = etss_get_view_row (etss, row);
 	if (view_row != -1)
 		e_table_model_row_changed (E_TABLE_MODEL (etss), view_row);
+	else
+		e_table_model_no_change (E_TABLE_MODEL (etss));
 }
 
 static void
@@ -273,6 +313,20 @@ etss_proxy_model_cell_changed_real (ETableSubset *etss, ETableModel *etm, int co
 	int view_row = etss_get_view_row (etss, row);
 	if (view_row != -1)
 		e_table_model_cell_changed (E_TABLE_MODEL (etss), col, view_row);
+	else
+		e_table_model_no_change (E_TABLE_MODEL (etss));
+}
+
+static void
+etss_proxy_model_rows_inserted_real (ETableSubset *etss, ETableModel *etm, int row, int count)
+{
+	e_table_model_no_change (E_TABLE_MODEL (etss));
+}
+
+static void
+etss_proxy_model_rows_deleted_real (ETableSubset *etss, ETableModel *etm, int row, int count)
+{
+	e_table_model_no_change (E_TABLE_MODEL (etss));
 }
 
 static void
@@ -282,6 +336,13 @@ etss_proxy_model_pre_change (ETableModel *etm, ETableSubset *etss)
 		(ETSS_CLASS(etss)->proxy_model_pre_change) (etss, etm);
 }
 
+static void
+etss_proxy_model_no_change (ETableModel *etm, ETableSubset *etss)
+{
+	if (ETSS_CLASS(etss)->proxy_model_no_change)
+		(ETSS_CLASS(etss)->proxy_model_no_change) (etss, etm);
+}
+
 static void
 etss_proxy_model_changed (ETableModel *etm, ETableSubset *etss)
 {
@@ -338,18 +399,20 @@ e_table_subset_construct (ETableSubset *etss, ETableModel *source, int nvals)
 	for (i = 0; i < nvals; i++)
 		etss->map_table [i] = i;
 
-	etss->table_model_pre_change_id = gtk_signal_connect (GTK_OBJECT (source), "model_pre_change",
-							      GTK_SIGNAL_FUNC (etss_proxy_model_pre_change), etss);
-	etss->table_model_changed_id = gtk_signal_connect (GTK_OBJECT (source), "model_changed",
-						     GTK_SIGNAL_FUNC (etss_proxy_model_changed), etss);
-	etss->table_model_row_changed_id = gtk_signal_connect (GTK_OBJECT (source), "model_row_changed",
-							 GTK_SIGNAL_FUNC (etss_proxy_model_row_changed), etss);
-	etss->table_model_cell_changed_id = gtk_signal_connect (GTK_OBJECT (source), "model_cell_changed",
-							  GTK_SIGNAL_FUNC (etss_proxy_model_cell_changed), etss);
+	etss->table_model_pre_change_id    = gtk_signal_connect (GTK_OBJECT (source), "model_pre_change",
+								 GTK_SIGNAL_FUNC (etss_proxy_model_pre_change), etss);
+	etss->table_model_no_change_id     = gtk_signal_connect (GTK_OBJECT (source), "model_no_change",
+								 GTK_SIGNAL_FUNC (etss_proxy_model_no_change), etss);
+	etss->table_model_changed_id       = gtk_signal_connect (GTK_OBJECT (source), "model_changed",
+								 GTK_SIGNAL_FUNC (etss_proxy_model_changed), etss);
+	etss->table_model_row_changed_id   = gtk_signal_connect (GTK_OBJECT (source), "model_row_changed",
+								 GTK_SIGNAL_FUNC (etss_proxy_model_row_changed), etss);
+	etss->table_model_cell_changed_id  = gtk_signal_connect (GTK_OBJECT (source), "model_cell_changed",
+								 GTK_SIGNAL_FUNC (etss_proxy_model_cell_changed), etss);
 	etss->table_model_rows_inserted_id = gtk_signal_connect (GTK_OBJECT (source), "model_rows_inserted",
-								GTK_SIGNAL_FUNC (etss_proxy_model_rows_inserted), etss);
-	etss->table_model_rows_deleted_id = gtk_signal_connect (GTK_OBJECT (source), "model_rows_deleted",
-							       GTK_SIGNAL_FUNC (etss_proxy_model_rows_deleted), etss);
+								 GTK_SIGNAL_FUNC (etss_proxy_model_rows_inserted), etss);
+	etss->table_model_rows_deleted_id  = gtk_signal_connect (GTK_OBJECT (source), "model_rows_deleted",
+								 GTK_SIGNAL_FUNC (etss_proxy_model_rows_deleted), etss);
 	
 	return E_TABLE_MODEL (etss);
 }
