@@ -1,20 +1,25 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* ETextModel - Text item model for evolution.
- * Copyright (C) 2000 Helix Code, Inc.
+/* 
+ * e-text-model.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author: Chris Lahey <clahey@umich.edu>
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * A majority of code taken from:
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * Text item type for GnomeCanvas widget
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * GnomeCanvas is basically a port of the Tk toolkit's most excellent
- * canvas widget.  Tk is copyrighted by the Regents of the University
- * of California, Sun Microsystems, and other parties.
- *
- * Copyright (C) 1998 The Free Software Foundation
- *
- * Author: Federico Mena <federico@nuclecu.unam.mx> */
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
 
 #undef  PARANOID_DEBUGGING
 
@@ -24,9 +29,12 @@
 #include <gtk/gtksignal.h>
 #include "e-text-model-repos.h"
 #include "e-text-model.h"
+#include "gal/util/e-util.h"
 
 #define CLASS(obj) (E_TEXT_MODEL_CLASS (GTK_OBJECT (obj)->klass))
 
+#define MAX_LENGTH (2047)
+
 enum {
 	E_TEXT_MODEL_CHANGED,
 	E_TEXT_MODEL_REPOSITION,
@@ -103,7 +111,7 @@ e_text_model_class_init (ETextModelClass *klass)
 	e_text_model_signals[E_TEXT_MODEL_CHANGED] =
 		gtk_signal_new ("changed",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETextModelClass, changed),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -111,7 +119,7 @@ e_text_model_class_init (ETextModelClass *klass)
 	e_text_model_signals[E_TEXT_MODEL_REPOSITION] =
 		gtk_signal_new ("reposition",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETextModelClass, reposition),
 				gtk_marshal_NONE__POINTER_POINTER,
 				GTK_TYPE_NONE, 2,
@@ -120,7 +128,7 @@ e_text_model_class_init (ETextModelClass *klass)
 	e_text_model_signals[E_TEXT_MODEL_OBJECT_ACTIVATED] =
 		gtk_signal_new ("object_activated",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETextModelClass, object_activated),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1,
@@ -129,12 +137,12 @@ e_text_model_class_init (ETextModelClass *klass)
 	e_text_model_signals[E_TEXT_MODEL_CANCEL_COMPLETION] =
 		gtk_signal_new ("cancel_completion",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETextModelClass, cancel_completion),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 	
-	gtk_object_class_add_signals (object_class, e_text_model_signals, E_TEXT_MODEL_LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_text_model_signals, E_TEXT_MODEL_LAST_SIGNAL);
 	
 	/* No default signal handlers. */
 	klass->changed          = NULL;
@@ -235,7 +243,7 @@ e_text_model_real_set_text (ETextModel *model, const gchar *text)
 	} else if (model->priv->text == NULL || strcmp (model->priv->text, text)) {
 		
 		g_free (model->priv->text);
-		model->priv->text = g_strdup (text);
+		model->priv->text = g_strndup (text, MAX_LENGTH);
 		model->priv->len = -1;
 
 		changed = TRUE;
@@ -254,10 +262,23 @@ e_text_model_real_insert (ETextModel *model, gint position, const gchar *text)
 {
 	EReposInsertShift repos;
 	gchar *new_text;
-	gint ins_len;
+	gint length;
 
-	new_text = g_strdup_printf ("%.*s%s%s", position, model->priv->text, text, model->priv->text + position);
-	ins_len = strlen (text);
+	if (model->priv->len < 0)
+		e_text_model_real_get_text_length (model);
+	length = strlen(text);
+
+	if (length + model->priv->len > MAX_LENGTH)
+		length = MAX_LENGTH - model->priv->len;
+	if (length <= 0)
+		return;
+
+	/* Can't use g_strdup_printf here because on some systems
+           printf ("%.*s"); is locale dependent. */
+	new_text = e_strdup_append_strings (model->priv->text, position,
+					    text, length,
+					    model->priv->text + position, -1,
+					    NULL);
 
 	if (model->priv->text)
 		g_free (model->priv->text);
@@ -265,13 +286,13 @@ e_text_model_real_insert (ETextModel *model, gint position, const gchar *text)
 	model->priv->text = new_text;
 	
 	if (model->priv->len >= 0)
-		model->priv->len += ins_len;
+		model->priv->len += length;
 	
 	e_text_model_changed (model);
 
 	repos.model = model;
 	repos.pos = position;
-	repos.len = ins_len;
+	repos.len = length;
 
 	e_text_model_reposition (model, e_repos_insert_shift, &repos);
 }
@@ -280,7 +301,22 @@ static void
 e_text_model_real_insert_length (ETextModel *model, gint position, const gchar *text, gint length)
 {
 	EReposInsertShift repos;
-	gchar *new_text = g_strdup_printf ("%.*s%.*s%s", position, model->priv->text, length, text, model->priv->text + position);
+	gchar *new_text;
+
+	if (model->priv->len < 0)
+		e_text_model_real_get_text_length (model);
+
+	if (length + model->priv->len > MAX_LENGTH)
+		length = MAX_LENGTH - model->priv->len;
+	if (length <= 0)
+		return;
+
+	/* Can't use g_strdup_printf here because on some systems
+           printf ("%.*s"); is locale dependent. */
+	new_text = e_strdup_append_strings (model->priv->text, position,
+					    text, length,
+					    model->priv->text + position, -1,
+					    NULL);
 
 	if (model->priv->text)
 		g_free (model->priv->text);
