@@ -1,25 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-
 /* 
- * Author : 
- *  Damon Chaplin <damon@helixcode.com>
+ * e-icon-bar.c
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Copyright 1999, 2000, 2001 Helix Code, Inc.
+ * Authors:
+ *   Damon Chaplin <damon@ximian.com>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 /*
@@ -31,6 +30,7 @@
 
 #include "e-icon-bar.h"
 
+#include <gtk/gtkdnd.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
@@ -39,6 +39,7 @@
 #include <gal/e-text/e-text.h>
 #include <gal/widgets/e-canvas-utils.h>
 #include <libgnomeui/gnome-canvas-rect-ellipse.h>
+#include "gal/util/e-util.h"
 
 /* These are the offsets of the icons & text in both views. Note that the
    shadow around icons is drawn in the space between items (as is the
@@ -185,7 +186,7 @@ e_icon_bar_class_init (EIconBarClass *class)
 	e_icon_bar_signals[ITEM_SELECTED] =
 		gtk_signal_new ("item_selected",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EIconBarClass,
 						   selected_item),
 				gtk_marshal_NONE__POINTER_INT,
@@ -194,14 +195,14 @@ e_icon_bar_class_init (EIconBarClass *class)
 	e_icon_bar_signals[ITEM_DRAGGED] =
 		gtk_signal_new ("item_dragged",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EIconBarClass,
 						   dragged_item),
 				gtk_marshal_NONE__POINTER_INT,
 				GTK_TYPE_NONE, 2, GTK_TYPE_GDK_EVENT,
 				GTK_TYPE_INT);
 
-	gtk_object_class_add_signals (object_class, e_icon_bar_signals,
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_icon_bar_signals,
 				      LAST_SIGNAL);
 
 	/* Method override */
@@ -270,7 +271,11 @@ e_icon_bar_init (EIconBar *icon_bar)
 					     E_ICON_BAR_COLOR_LAST, FALSE,
 					     TRUE, success);
 	if (nfailed)
-		g_warning ("Failed to allocate all colors");
+		g_warning ("EIconBar failed to allocate all colors");
+
+	gtk_drag_dest_set (GTK_WIDGET (icon_bar),
+			   0, NULL, 0,
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 }
 
 
@@ -323,9 +328,7 @@ e_icon_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
 	EIconBar *icon_bar;
 	gint canvas_width, canvas_height, height;
-#if 0
-	g_print ("In e_icon_bar_size_allocate\n");
-#endif
+
 	icon_bar = E_ICON_BAR (widget);
 
 	GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
@@ -677,7 +680,8 @@ e_icon_bar_add_item (EIconBar	    *icon_bar,
 			    icon_bar);
 
 	item.pixbuf = image;
-	gdk_pixbuf_ref (image);
+	if (image)
+		gdk_pixbuf_ref (image);
 
 	flattened = flatten_alpha (image,
 				   rgb_from_gdk_color (&style->bg [GTK_STATE_NORMAL]));
@@ -979,7 +983,6 @@ static void
 e_icon_bar_reflow (ECanvas *canvas)
 {
 #if 0
-	g_print ("In e_icon_bar_on_canvas_reflow\n");
 	gtk_widget_queue_resize (GTK_WIDGET (canvas));
 #endif
 }
@@ -1092,9 +1095,6 @@ e_icon_bar_item_pressed (EIconBar *icon_bar,
 
 	if (button == 1 && item_num != -1) {
 		icon_bar->mouse_over_item_num = item_num;
-#if 0
-		g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
-#endif
 		icon_bar->pressed_item_num = item_num;
 		icon_bar->pressed_x = event->button.x;
 		icon_bar->pressed_y = event->button.y;
@@ -1150,10 +1150,6 @@ e_icon_bar_item_motion (EIconBar *icon_bar,
 {
 	gboolean need_redraw = TRUE;
 
-#if 0
-	g_print ("In e_icon_bar_item_motion\n");
-#endif
-
 	if (event && event->motion.state & GDK_BUTTON1_MASK
 	    && icon_bar->pressed_item_num != -1) {
 		if (icon_bar->enable_drags
@@ -1192,9 +1188,6 @@ e_icon_bar_item_motion (EIconBar *icon_bar,
 	}
 
 	icon_bar->mouse_over_item_num = item_num;
-#if 0
-	g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
-#endif
 
 	if (need_redraw)
 		gtk_widget_queue_draw (GTK_WIDGET (icon_bar));
@@ -1552,7 +1545,9 @@ e_icon_bar_drag_motion (GtkWidget      *widget,
 			guint           time)
 {
 	EIconBar *icon_bar;
-	gint item_num, before_item, scroll_x, scroll_y;
+	GdkDragAction action;
+	gint item_num, before_item;
+	gint scroll_x, scroll_y;
 
 	g_return_val_if_fail (E_IS_ICON_BAR (widget), FALSE);
 
@@ -1570,6 +1565,8 @@ e_icon_bar_drag_motion (GtkWidget      *widget,
 	e_icon_bar_item_motion (icon_bar, item_num, NULL);
 	e_icon_bar_set_dragging_before_item (icon_bar, before_item);
 
+	action = 0;
+
 	/* Check if the mouse is at the top or bottom of the bar, and if it is
 	   scroll up/down. */
 	if (y < E_ICON_BAR_DRAG_AUTO_SCROLL_OFFSET)
@@ -1581,7 +1578,7 @@ e_icon_bar_drag_motion (GtkWidget      *widget,
 			gtk_timeout_remove (icon_bar->auto_scroll_timeout_id);
 			icon_bar->auto_scroll_timeout_id = 0;
 		}
-		return FALSE;
+		return TRUE;
 	}
 
 	if (icon_bar->auto_scroll_timeout_id == 0) {
@@ -1589,7 +1586,7 @@ e_icon_bar_drag_motion (GtkWidget      *widget,
 		icon_bar->auto_scroll_delay = E_ICON_BAR_SCROLL_DELAY;
 	}
 
-	return FALSE;
+	return TRUE;
 }
 
 
@@ -1613,9 +1610,6 @@ e_icon_bar_drag_leave (GtkWidget      *widget,
 
 	if (icon_bar->mouse_over_item_num != -1) {
 		icon_bar->mouse_over_item_num = -1;
-#if 0
-		g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
-#endif
 		gtk_widget_queue_draw (GTK_WIDGET (icon_bar));
 	}
 }
