@@ -1,13 +1,32 @@
-/*
- * color-palette.c : A color selector palette
- * 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * color-palette.c - A color selector palette
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
  * This code was extracted from widget-color-combo.c
  *   written by Miguel de Icaza (miguel@kernel.org) and
  *   Dom Lachowicz (dominicl@seas.upenn.edu). The extracted
  *   code was re-packaged into a separate object by
  *   Michael Levy (mlevy@genoscope.cns.fr)
+ *   And later revised and polished by
+ *   Almer S. Tigelaar (almer@gnome.org)
  *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtksignal.h>
@@ -15,6 +34,7 @@
 #include <libgnomeui/gnome-canvas.h>
 #include <libgnomeui/gnome-canvas-rect-ellipse.h>
 #include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
 #include "color-group.h"
 #include "color-palette.h"
 #include "e-colors.h"
@@ -27,12 +47,12 @@ enum {
 	LAST_SIGNAL
 };
 
-typedef struct {
+struct _ColorNamePair {
 	char *color;	/* rgb color or otherwise - eg. "rgb:FF/FF/FF" */
 	char *name;	/* english name - eg. "white" */
-} ColorNamePair;
+};
 
-static gint color_palette_signals [LAST_SIGNAL] = { 0, };
+static guint color_palette_signals [LAST_SIGNAL] = { 0, };
 
 static GtkObjectClass *color_palette_parent_class;
 
@@ -42,38 +62,40 @@ static void
 color_palette_finalize (GtkObject *object)
 {
 	ColorPalette *P = COLOR_PALETTE (object);
-	
+
 	if (P->tool_tip) {
 		gtk_object_unref (GTK_OBJECT (P->tool_tip));
 		P->tool_tip = NULL;
 	}
-	
+
 	g_free (P->items);
 
-	if (P->color_group) {
-		gtk_object_unref(GTK_OBJECT (P->color_group));
-		P->color_group = NULL;
-	}
+	if (P->current_color)
+		gdk_color_free (P->current_color);
 	
+	color_palette_set_group (P, NULL);
+
 	(*color_palette_parent_class->finalize) (object);
 }
 
 
-typedef void (*GtkSignal_NONE__POINTER_BOOL) (GtkObject * object,
-					      gpointer arg1,
-					      gboolean arg2,
-					      gpointer user_data);
-static void 
-marshal_NONE__POINTER_BOOL (GtkObject * object,
-			    GtkSignalFunc func,
-			    gpointer func_data,
-			    GtkArg * args)
+typedef void (*GtkSignal_NONE__POINTER_BOOL_BOOL) (GtkObject * object,
+						   gpointer arg1,
+						   gboolean arg2,
+						   gboolean arg3,
+						   gpointer user_data);
+static void
+marshal_NONE__POINTER_BOOL_BOOL (GtkObject * object,
+				 GtkSignalFunc func,
+				 gpointer func_data,
+				 GtkArg * args)
 {
-	GtkSignal_NONE__POINTER_BOOL rfunc;
-	rfunc = (GtkSignal_NONE__POINTER_BOOL) func;
+	GtkSignal_NONE__POINTER_BOOL_BOOL rfunc;
+	rfunc = (GtkSignal_NONE__POINTER_BOOL_BOOL) func;
 	(*rfunc) (object,
 		  GTK_VALUE_POINTER (args[0]),
 		  GTK_VALUE_BOOL    (args[1]),
+		  GTK_VALUE_BOOL    (args[2]),
 		  func_data);
 }
 
@@ -82,26 +104,27 @@ static void
 color_palette_class_init (GtkObjectClass *object_class)
 {
 	object_class->finalize = color_palette_finalize;
-	
+
 	color_palette_parent_class = gtk_type_class (gtk_vbox_get_type ());
-	
+
 	color_palette_signals [CHANGED] =
 		gtk_signal_new (
 			"changed",
 			GTK_RUN_LAST,
-			object_class->type,
+			E_OBJECT_CLASS_TYPE (object_class),
 			GTK_SIGNAL_OFFSET (ColorPaletteClass, changed),
-			marshal_NONE__POINTER_BOOL,
-			GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_BOOL);
-	
-	gtk_object_class_add_signals (object_class, color_palette_signals, LAST_SIGNAL);
+			marshal_NONE__POINTER_BOOL_BOOL,
+			GTK_TYPE_NONE, 3, GTK_TYPE_POINTER,
+			GTK_TYPE_BOOL, GTK_TYPE_BOOL);
+
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, color_palette_signals, LAST_SIGNAL);
 }
 
 GtkType
 color_palette_get_type (void)
 {
 	static GtkType type = 0;
-	
+
 	if (!type){
 		GtkTypeInfo info = {
 			"ColorPalette",
@@ -113,10 +136,10 @@ color_palette_get_type (void)
 			NULL, /* reserved 2 */
 			(GtkClassInitFunc) NULL
 		};
-		
+
 		type = gtk_type_unique (gtk_vbox_get_type (), &info);
 	}
-	
+
 	return type;
 }
 
@@ -126,14 +149,22 @@ color_palette_get_type (void)
  * prop
  */
 static void
-emit_change (ColorPalette *P, GdkColor *color, gboolean custom, gboolean tell_group)
+emit_change (ColorPalette *P, GdkColor *color, gboolean custom, gboolean by_user)
 {
-	if (tell_group)
-		color_group_add_color (P->color_group, color, custom);
-	else
-		gtk_signal_emit (
-			GTK_OBJECT (P), color_palette_signals [CHANGED], color, custom);
+	GdkColor *new;
 	
+	/* Set it as current color */
+	if (P->current_color)
+		gdk_color_free (P->current_color);
+	new = make_color (P, color);
+	P->current_color = new ? gdk_color_copy (new) : NULL;
+
+	/* Only add custom colors to the group */
+	if (custom && color)
+		color_group_add_color (P->color_group, color);
+	
+	gtk_signal_emit (
+		GTK_OBJECT (P), color_palette_signals [CHANGED], color, custom, by_user);
 }
 
 
@@ -144,7 +175,7 @@ emit_change (ColorPalette *P, GdkColor *color, gboolean custom, gboolean tell_gr
  * Also take care of setting up the GnomeColorPicker 'display'
  */
 static void
-color_palette_change_custom_color (ColorPalette *P, GdkColor *new)
+color_palette_change_custom_color (ColorPalette *P, GdkColor const * const new)
 {
 	int index;
 	GnomeCanvasItem *item;
@@ -157,13 +188,13 @@ color_palette_change_custom_color (ColorPalette *P, GdkColor *new)
 	/* make sure there is room */
 	if (P->custom_color_pos == -1)
 		return;
-	
+
 	for (index = P->custom_color_pos; index < P->total - 1; index++) {
 		GdkColor *color;
 		GdkColor *outline;
 		item = P->items [index];
 		next_item = P->items [index + 1];
-		
+
 		gtk_object_get (GTK_OBJECT (next_item),
 				"fill_color_gdk", &color,
 				"outline_color_gdk", &outline,
@@ -198,7 +229,7 @@ cust_color_set (GtkWidget  *color_picker, guint r, guint g, guint b, guint a,
 {
 	GdkColor *c_color;
 
-	c_color =  g_new (GdkColor, 1);
+	c_color = g_new (GdkColor, 1);
 
 	c_color->red   = (gushort)r;
 	c_color->green = (gushort)g;
@@ -228,39 +259,61 @@ color_clicked (GtkWidget *button, ColorPalette *P)
 	int              index;
 	GnomeCanvasItem *item;
 	GdkColor        *gdk_color;
-	
+
 	index = GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (button)));
 	item  = P->items [index];
-	
+
 	gtk_object_get (
 		GTK_OBJECT (item),
 		"fill_color_gdk", &gdk_color,
 		NULL);
-	
+
 	emit_change (P, gdk_color, FALSE, TRUE);
 
 	g_free (gdk_color);
 }
 
 /*
-  The color group sent the 'color_changed' signal
-*/
+ * The color group sent the 'custom_color_add' signal
+ */
 static void
-cb_group_color_change (GtkObject *cg, GdkColor * color, gboolean custom, ColorPalette *P)
+cb_group_custom_color_add (GtkObject *cg, GdkColor *color, ColorPalette *P)
 {
-	if (custom) {
-		GdkColor *new;
-		
-		new = make_color (P, color);
-		color_palette_change_custom_color (P, new);
-	}
-	/* 
-	   We set the last parameter to FALSE in order to avoid telling the 
-	   group something it already knows (and causing a loop)
-	 */
-	emit_change(P, color, custom, FALSE);
+	GdkColor *new;
+	
+	new = make_color (P, color);
+	color_palette_change_custom_color (P, new);
 }
 
+/*
+ * Find out if a color is in the default palette (not in the custom colors!)
+ *
+ * Utility function
+ */
+static gboolean
+color_in_palette (ColorNamePair *set, GdkColor *color)
+{
+	int i;
+
+	g_return_val_if_fail (set != NULL, FALSE);
+       
+	if (color == NULL)
+		return TRUE;
+		
+	/* Iterator over all the colors and try to find
+	 * if we can find @color
+	 */
+	for (i = 0; set[i].color != NULL; i++) {
+		GdkColor current;
+		
+		gdk_color_parse (set[i].color, &current);
+		
+		if (gdk_color_equal (color, &current))
+			return TRUE;
+	}
+
+	return FALSE;
+}
 
 /*
  * Create the individual color buttons
@@ -268,27 +321,27 @@ cb_group_color_change (GtkObject *cg, GdkColor * color, gboolean custom, ColorPa
  * Utility function
  */
 static GnomeCanvasItem *
-color_palette_button_new(ColorPalette *P, GtkTable* table, 
-			 GtkTooltips *tool_tip, ColorNamePair* color_name, 
+color_palette_button_new(ColorPalette *P, GtkTable* table,
+			 GtkTooltips *tool_tip, ColorNamePair* color_name,
 			 gint col, gint row, int data)
 {
         GtkWidget *button;
 	GtkWidget *canvas;
 	GnomeCanvasItem *item;
-	
+
 	button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	
+
 	gtk_widget_push_visual (gdk_imlib_get_visual ());
 	gtk_widget_push_colormap (gdk_imlib_get_colormap ());
 	canvas = gnome_canvas_new ();
 	gtk_widget_pop_colormap ();
 	gtk_widget_pop_visual ();
-	
+
 	gtk_widget_set_usize (canvas, COLOR_PREVIEW_WIDTH, COLOR_PREVIEW_HEIGHT);
 	gtk_container_add (GTK_CONTAINER (button), canvas);
-	
-	item  = gnome_canvas_item_new (GNOME_CANVAS_GROUP (gnome_canvas_root 
+
+	item  = gnome_canvas_item_new (GNOME_CANVAS_GROUP (gnome_canvas_root
 							   (GNOME_CANVAS (canvas))),
 				       gnome_canvas_rect_get_type (),
 				       "x1", 0.0,
@@ -297,13 +350,13 @@ color_palette_button_new(ColorPalette *P, GtkTable* table,
 				       "y2", (double) COLOR_PREVIEW_HEIGHT,
 				       "fill_color", color_name->color,
 				       NULL);
-	
+
 	gtk_tooltips_set_tip (tool_tip, button, _(color_name->name),
 			      "Private+Unused");
-	
+
 	gtk_table_attach (table, button,
 			  col, col+1, row, row+1, GTK_FILL, GTK_FILL, 1, 1);
-	
+
 	gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC(color_clicked), P);
 	gtk_object_set_user_data (GTK_OBJECT (button),
@@ -311,34 +364,35 @@ color_palette_button_new(ColorPalette *P, GtkTable* table,
 	return item;
 }
 
+static void
+cb_custom_colors (GdkColor const * const color, gpointer data)
+{
+	ColorPalette *P = data;
+	
+	if (color)
+		color_palette_change_custom_color (P, color);
+}
+
 /*
-  gets history information from the group
-*/
+ * gets history information from the group
+ */
 static void
 custom_color_history_setup(ColorPalette *P)
 {
-	GdkColor *color;
-	
 	g_return_if_fail (P != NULL);
 	g_return_if_fail (P->color_group != NULL);
-	
-	for (color = color_group_oldest_color (P->color_group);
-	     color != NULL ;
-	     color = color_group_previous_color(P->color_group)) 
-	    color_palette_change_custom_color (P , color);
 
-	return;
+	/* Sync our own palette with all the custom colors in the group */
+	color_group_get_custom_colors (P->color_group, (CbCustomColors) cb_custom_colors, P);
 }
 
-
-
 /*
  * Creates the color table
  */
 static GtkWidget *
-color_palette_setup (ColorPalette *P, 
-		     char const * const no_color_label, 
-		     int ncols, int nrows, 
+color_palette_setup (ColorPalette *P,
+		     char const * const no_color_label,
+		     int ncols, int nrows,
 		     ColorNamePair *color_names)
 {
 	GtkWidget *nocolor_button;
@@ -351,7 +405,7 @@ color_palette_setup (ColorPalette *P,
 
 
 	if (no_color_label != NULL) {
-	        nocolor_button = gtk_button_new_with_label (no_color_label);
+		nocolor_button = gtk_button_new_with_label (no_color_label);
 
 		gtk_table_attach (GTK_TABLE (table), nocolor_button,
 				  0, ncols, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);
@@ -362,11 +416,11 @@ color_palette_setup (ColorPalette *P,
 	P->tool_tip = tool_tip = gtk_tooltips_new();
 	P->custom_color_pos = -1;
 	total = 0;
-	
+
 	for (row = 0; row < nrows; row++) {
 		for (col = 0; col < ncols; col++) {
 			int pos;
-			
+
 			pos = row * ncols + col;
 			/*
 			 * If we are done with all of the colors in color_names
@@ -398,7 +452,7 @@ color_palette_setup (ColorPalette *P,
 				row = nrows;
 				break;
 			}
-			
+
 			P->items [total] =
 				color_palette_button_new (
 					P,
@@ -412,8 +466,8 @@ color_palette_setup (ColorPalette *P,
 		}
 	}
 	P->total = total;
-	
-	
+
+
 	/* "Custom" color - we'll pop up a GnomeColorPicker */
 	cust_label = gtk_label_new (_("Custom Color:"));
 	gtk_table_attach (GTK_TABLE (table), cust_label, 0, ncols - 3 ,
@@ -431,13 +485,22 @@ color_palette_setup (ColorPalette *P,
 	return table;
 }
 
+void
+color_palette_set_current_color (ColorPalette *P, GdkColor *color)
+{
+	g_return_if_fail (P != NULL);
+	g_return_if_fail (IS_COLOR_GROUP (P->color_group));
+
+	emit_change (P, color, color_in_palette (P->default_set, color), FALSE);
+}
+
 GdkColor *
 color_palette_get_current_color(ColorPalette *P)
 {
 	g_return_val_if_fail (P != NULL, NULL);
 	g_return_val_if_fail (IS_COLOR_GROUP (P->color_group), NULL);
 
-	return color_group_get_current_color(P->color_group);
+	return P->current_color ? gdk_color_copy (P->current_color) : NULL;
 }
 
 GtkWidget *
@@ -455,26 +518,22 @@ color_palette_get_color_picker (ColorPalette *P)
 static void
 color_palette_construct (ColorPalette *P,
 			 char const * const no_color_label,
-			 int ncols, int nrows, ColorNamePair *color_names)
+			 int ncols, int nrows)
 {
 	GtkWidget * table;
 	g_return_if_fail (P != NULL);
 	g_return_if_fail (IS_COLOR_PALETTE (P));
-	g_return_if_fail (color_names != NULL);
-	
+
 	P->items = g_malloc (sizeof (GnomeCanvasItem *) * ncols * nrows);
-	
+
 	/*
 	 * Our table selector
 	 */
-	table = color_palette_setup (P, no_color_label, ncols, 
-				     nrows, color_names);
+	table = color_palette_setup (P, no_color_label, ncols,
+				     nrows, P->default_set);
 	gtk_container_add (GTK_CONTAINER(P), table);
 }
 
-
-
-
 /*
  * More verbose constructor. Allows for specifying the rows, columns, and
  * Colors this palette will contain
@@ -486,27 +545,48 @@ color_palette_construct (ColorPalette *P,
 static GtkWidget*
 color_palette_new_with_vals (char const * const no_color_label,
 			     int ncols, int nrows, ColorNamePair *color_names,
-			     GdkColor *default_color, 
-			     const gchar *group_name){
+			     GdkColor *default_color,
+			     ColorGroup *cg)
+{
 	ColorPalette *P;
-	
-	g_return_val_if_fail (color_names != NULL, NULL);
-	
-	P = gtk_type_new (color_palette_get_type ());
-	
-	P->default_color = default_color;
-	P->color_group = COLOR_GROUP (color_group_new_named(group_name));
 
-	color_palette_construct (P, no_color_label, ncols, nrows, color_names);
+	g_return_val_if_fail (color_names != NULL, NULL);
+
+	P = gtk_type_new (color_palette_get_type ());
+
+	P->default_set   = color_names;
+	P->default_color = default_color;
+	P->current_color = default_color ? gdk_color_copy (default_color) : NULL;
+	color_palette_set_group (P, cg);
+
+	color_palette_construct (P, no_color_label, ncols, nrows);
 	custom_color_history_setup(P);
-	
-	gtk_signal_connect (GTK_OBJECT(P->color_group), "color_change",
-			    GTK_SIGNAL_FUNC (cb_group_color_change),
-			    P);
-	
+
 	return GTK_WIDGET (P);
 }
 
+void
+color_palette_set_group (ColorPalette *P, ColorGroup *cg)
+{
+	if (P->color_group == cg)
+		return;
+
+	if (P->color_group) {
+		gtk_signal_disconnect_by_func (
+			GTK_OBJECT (P->color_group),
+			GTK_SIGNAL_FUNC (cb_group_custom_color_add),
+			P);
+		gtk_object_unref(GTK_OBJECT (P->color_group));
+		P->color_group = NULL;
+	}
+	if (cg != NULL) {
+		P->color_group = COLOR_GROUP (cg);
+		gtk_signal_connect (GTK_OBJECT (cg), "custom_color_add",
+			GTK_SIGNAL_FUNC (cb_group_custom_color_add),
+			P);
+
+	}
+}
 
 static ColorNamePair default_color_set [] = {
 	{"rgb:0/0/0", N_("black")},
@@ -591,12 +671,11 @@ static ColorNamePair default_color_set [] = {
  */
 GtkWidget*
 color_palette_new (const char *no_color_label,
-		   GdkColor *default_color, const gchar *group_name)
+		   GdkColor *default_color, ColorGroup *color_group)
 {
 	/* specify 6 rows to allow for a row of custom colors */
 	return color_palette_new_with_vals (no_color_label,
-					    8, 6, 
+					    8, 6,
 					    default_color_set, default_color,
-					    group_name);
+					    color_group);
 }
-
