@@ -53,7 +53,7 @@ enum {
 	ARG_WIDTH,
 	ARG_MINIMUM_WIDTH,
 	ARG_HEIGHT,
-	ARG_SPACING,
+	ARG_SPACING
 };
 
 GtkType
