@@ -1,27 +1,31 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
+/* 
  * e-canvas.c
- * Copyright (C) 2000  Helix Code, Inc.
- * Author: Chris Lahey <clahey@helixcode.com>
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <gtk/gtksignal.h>
 #include "e-canvas.h"
+#include "gal/util/e-util.h"
+#include <X11/Xlib.h>
+#include <gtk/gtkmain.h>
 
 static void e_canvas_init           (ECanvas         *card);
 static void e_canvas_destroy        (GtkObject        *object);
@@ -30,6 +34,8 @@ static void e_canvas_realize        (GtkWidget        *widget);
 static void e_canvas_unrealize      (GtkWidget        *widget);
 static gint e_canvas_key            (GtkWidget        *widget,
 				     GdkEventKey      *event);
+static gint e_canvas_button         (GtkWidget        *widget,
+				     GdkEventButton   *event);
 
 static gint e_canvas_visibility     (GtkWidget        *widget,
 				     GdkEventVisibility *event,
@@ -40,10 +46,15 @@ static gint e_canvas_focus_in       (GtkWidget        *widget,
 static gint e_canvas_focus_out      (GtkWidget        *widget,
 				     GdkEventFocus    *event);
 
+static void e_canvas_style_set      (GtkWidget        *widget,
+				     GtkStyle         *previous_style);
+
 static int emit_event (GnomeCanvas *canvas, GdkEvent *event);
 
 static GnomeCanvasClass *parent_class = NULL;
 
+#define d(x)
+
 enum {
 	REFLOW,
 	LAST_SIGNAL
@@ -83,32 +94,35 @@ e_canvas_class_init (ECanvasClass *klass)
 	GnomeCanvasClass *canvas_class;
 	GtkWidgetClass *widget_class;
 
-	object_class = (GtkObjectClass*) klass;
-	canvas_class = (GnomeCanvasClass *) klass;
-	widget_class = (GtkWidgetClass *) klass;
+	object_class                       = (GtkObjectClass*) klass;
+	canvas_class                       = (GnomeCanvasClass *) klass;
+	widget_class                       = (GtkWidgetClass *) klass;
 
-	parent_class = gtk_type_class (gnome_canvas_get_type ());
+	parent_class                       = gtk_type_class (gnome_canvas_get_type ());
 
-	object_class->destroy = e_canvas_destroy;
+	object_class->destroy              = e_canvas_destroy;
 
-	widget_class->key_press_event = e_canvas_key;
-	widget_class->key_release_event = e_canvas_key;
-	widget_class->focus_in_event = e_canvas_focus_in;
-	widget_class->focus_out_event = e_canvas_focus_out;
-	widget_class->realize = e_canvas_realize;
-	widget_class->unrealize = e_canvas_unrealize;
+	widget_class->key_press_event      = e_canvas_key;
+	widget_class->key_release_event    = e_canvas_key;
+	widget_class->button_press_event   = e_canvas_button;
+	widget_class->button_release_event = e_canvas_button;
+	widget_class->focus_in_event       = e_canvas_focus_in;
+	widget_class->focus_out_event      = e_canvas_focus_out;
+	widget_class->style_set            = e_canvas_style_set;
+	widget_class->realize              = e_canvas_realize;
+	widget_class->unrealize            = e_canvas_unrealize;
 
-	klass->reflow = NULL;
+	klass->reflow                      = NULL;
 
 	e_canvas_signals [REFLOW] =
 		gtk_signal_new ("reflow",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECanvasClass, reflow),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
-	gtk_object_class_add_signals (object_class, e_canvas_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_canvas_signals, LAST_SIGNAL);
 }
 
 static void
@@ -128,6 +142,7 @@ e_canvas_destroy (GtkObject *object)
 
 	if (canvas->idle_id)
 		g_source_remove(canvas->idle_id);
+	canvas->idle_id = 0;
 
 	if (canvas->toplevel) {
 		if (canvas->visibility_notify_id)
@@ -139,6 +154,8 @@ e_canvas_destroy (GtkObject *object)
 		canvas->toplevel = NULL;
 	}
 
+	e_canvas_hide_tooltip(canvas);
+
 	if ((GTK_OBJECT_CLASS (parent_class))->destroy)
 		(*(GTK_OBJECT_CLASS (parent_class))->destroy) (object);
 }
@@ -150,17 +167,6 @@ e_canvas_new ()
 }
 
 
-/* Returns whether the item is an inferior of or is equal to the parent. */
-static int
-is_descendant (GnomeCanvasItem *item, GnomeCanvasItem *parent)
-{
-	for (; item; item = item->parent)
-		if (item == parent)
-			return TRUE;
-
-	return FALSE;
-}
-
 /* Emits an event for an item in the canvas, be it the current item, grabbed
  * item, or focused item, as appropriate.
  */
@@ -173,10 +179,18 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	GnomeCanvasItem *parent;
 	guint mask;
 
-	/* Perform checks for grabbed items */
+	/* Choose where we send the event */
 
-	if (canvas->grabbed_item && !is_descendant (canvas->current_item, canvas->grabbed_item))
-		return FALSE;
+	item = canvas->current_item;
+
+	if (canvas->focused_item
+	    && ((event->type == GDK_KEY_PRESS) || (event->type == GDK_KEY_RELEASE) || (event->type == GDK_FOCUS_CHANGE)))
+		item = canvas->focused_item;
+
+	if (canvas->grabbed_item)
+		item = canvas->grabbed_item;
+
+	/* Perform checks for grabbed items */
 
 	if (canvas->grabbed_item) {
 		switch (event->type) {
@@ -247,14 +261,6 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 		break;
 	}
 
-	/* Choose where we send the event */
-
-	item = canvas->current_item;
-
-	if (canvas->focused_item
-	    && ((event->type == GDK_KEY_PRESS) || (event->type == GDK_KEY_RELEASE) || (event->type == GDK_FOCUS_CHANGE)))
-		item = canvas->focused_item;
-
 	/* The event is propagated up the hierarchy (for if someone connected to
 	 * a group instead of a leaf event), and emission is stopped if a
 	 * handler returns TRUE, just like for GtkWidget events.
@@ -299,6 +305,264 @@ e_canvas_key (GtkWidget *widget, GdkEventKey *event)
 	return emit_event (canvas, &full_event);
 }
 
+
+/* This routine invokes the point method of the item.  The argument x, y should
+ * be in the parent's item-relative coordinate system.  This routine applies the
+ * inverse of the item's transform, maintaining the affine invariant.
+ */
+#define HACKISH_AFFINE
+
+static double
+gnome_canvas_item_invoke_point (GnomeCanvasItem *item, double x, double y, int cx, int cy,
+				GnomeCanvasItem **actual_item)
+{
+#ifdef HACKISH_AFFINE
+	double i2w[6], w2c[6], i2c[6], c2i[6];
+	ArtPoint c, i;
+#endif
+
+#ifdef HACKISH_AFFINE
+	gnome_canvas_item_i2w_affine (item, i2w);
+	gnome_canvas_w2c_affine (item->canvas, w2c);
+	art_affine_multiply (i2c, i2w, w2c);
+	art_affine_invert (c2i, i2c);
+	c.x = cx;
+	c.y = cy;
+	art_affine_point (&i, &c, c2i);
+	x = i.x;
+	y = i.y;
+#endif
+
+	return (* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->point) (
+		item, x, y, cx, cy, actual_item);
+}
+
+/* Re-picks the current item in the canvas, based on the event's coordinates.
+ * Also emits enter/leave events for items as appropriate.
+ */
+#define DISPLAY_X1(canvas) (GNOME_CANVAS (canvas)->layout.xoffset)
+#define DISPLAY_Y1(canvas) (GNOME_CANVAS (canvas)->layout.yoffset)
+static int
+pick_current_item (GnomeCanvas *canvas, GdkEvent *event)
+{
+	int button_down;
+	double x, y;
+	int cx, cy;
+	int retval;
+
+	retval = FALSE;
+
+	/* If a button is down, we'll perform enter and leave events on the
+	 * current item, but not enter on any other item.  This is more or less
+	 * like X pointer grabbing for canvas items.
+	 */
+	button_down = canvas->state & (GDK_BUTTON1_MASK
+				       | GDK_BUTTON2_MASK
+				       | GDK_BUTTON3_MASK
+				       | GDK_BUTTON4_MASK
+				       | GDK_BUTTON5_MASK);
+	d(g_print ("%s:%d: button_down = %s\n", __FUNCTION__, __LINE__, button_down ? "TRUE" : "FALSE"));
+	if (!button_down)
+		canvas->left_grabbed_item = FALSE;
+
+	/* Save the event in the canvas.  This is used to synthesize enter and
+	 * leave events in case the current item changes.  It is also used to
+	 * re-pick the current item if the current one gets deleted.  Also,
+	 * synthesize an enter event.
+	 */
+	if (event != &canvas->pick_event) {
+		if ((event->type == GDK_MOTION_NOTIFY) || (event->type == GDK_BUTTON_RELEASE)) {
+			/* these fields have the same offsets in both types of events */
+
+			canvas->pick_event.crossing.type       = GDK_ENTER_NOTIFY;
+			canvas->pick_event.crossing.window     = event->motion.window;
+			canvas->pick_event.crossing.send_event = event->motion.send_event;
+			canvas->pick_event.crossing.subwindow  = NULL;
+			canvas->pick_event.crossing.x          = event->motion.x;
+			canvas->pick_event.crossing.y          = event->motion.y;
+			canvas->pick_event.crossing.mode       = GDK_CROSSING_NORMAL;
+			canvas->pick_event.crossing.detail     = GDK_NOTIFY_NONLINEAR;
+			canvas->pick_event.crossing.focus      = FALSE;
+			canvas->pick_event.crossing.state      = event->motion.state;
+
+			/* these fields don't have the same offsets in both types of events */
+
+			if (event->type == GDK_MOTION_NOTIFY) {
+				canvas->pick_event.crossing.x_root = event->motion.x_root;
+				canvas->pick_event.crossing.y_root = event->motion.y_root;
+			} else {
+				canvas->pick_event.crossing.x_root = event->button.x_root;
+				canvas->pick_event.crossing.y_root = event->button.y_root;
+			}
+		} else
+			canvas->pick_event = *event;
+	}
+
+	/* Don't do anything else if this is a recursive call */
+
+	if (canvas->in_repick)
+		return retval;
+
+	/* LeaveNotify means that there is no current item, so we don't look for one */
+
+	if (canvas->pick_event.type != GDK_LEAVE_NOTIFY) {
+		/* these fields don't have the same offsets in both types of events */
+
+		if (canvas->pick_event.type == GDK_ENTER_NOTIFY) {
+			x = canvas->pick_event.crossing.x + DISPLAY_X1 (canvas) - canvas->zoom_xofs;
+			y = canvas->pick_event.crossing.y + DISPLAY_Y1 (canvas) - canvas->zoom_yofs;
+		} else {
+			x = canvas->pick_event.motion.x + DISPLAY_X1 (canvas) - canvas->zoom_xofs;
+			y = canvas->pick_event.motion.y + DISPLAY_Y1 (canvas) - canvas->zoom_yofs;
+		}
+
+		/* canvas pixel coords */
+
+		cx = (int) (x + 0.5);
+		cy = (int) (y + 0.5);
+
+		/* world coords */
+
+		x = canvas->scroll_x1 + x / canvas->pixels_per_unit;
+		y = canvas->scroll_y1 + y / canvas->pixels_per_unit;
+
+		/* find the closest item */
+
+		if (canvas->root->object.flags & GNOME_CANVAS_ITEM_VISIBLE)
+			gnome_canvas_item_invoke_point (canvas->root, x, y, cx, cy,
+							&canvas->new_current_item);
+		else
+			canvas->new_current_item = NULL;
+	} else
+		canvas->new_current_item = NULL;
+
+	if ((canvas->new_current_item == canvas->current_item) && !canvas->left_grabbed_item)
+		return retval; /* current item did not change */
+
+	/* Synthesize events for old and new current items */
+
+	if ((canvas->new_current_item != canvas->current_item)
+	    && (canvas->current_item != NULL)
+	    && !canvas->left_grabbed_item) {
+		GdkEvent new_event;
+		GnomeCanvasItem *item;
+
+		item = canvas->current_item;
+
+		new_event = canvas->pick_event;
+		new_event.type = GDK_LEAVE_NOTIFY;
+
+		new_event.crossing.detail = GDK_NOTIFY_ANCESTOR;
+		new_event.crossing.subwindow = NULL;
+		canvas->in_repick = TRUE;
+		retval = emit_event (canvas, &new_event);
+		canvas->in_repick = FALSE;
+	}
+
+	/* new_current_item may have been set to NULL during the call to emit_event() above */
+
+	if ((canvas->new_current_item != canvas->current_item) && button_down) {
+		canvas->left_grabbed_item = TRUE;
+		return retval;
+	}
+
+	/* Handle the rest of cases */
+
+	canvas->left_grabbed_item = FALSE;
+	canvas->current_item = canvas->new_current_item;
+
+	if (canvas->current_item != NULL) {
+		GdkEvent new_event;
+
+		new_event = canvas->pick_event;
+		new_event.type = GDK_ENTER_NOTIFY;
+		new_event.crossing.detail = GDK_NOTIFY_ANCESTOR;
+		new_event.crossing.subwindow = NULL;
+		retval = emit_event (canvas, &new_event);
+	}
+
+	return retval;
+}
+
+/* Button event handler for the canvas */
+static gint
+e_canvas_button (GtkWidget *widget, GdkEventButton *event)
+{
+	GnomeCanvas *canvas;
+	int mask;
+	int retval;
+
+	g_return_val_if_fail (widget != NULL, FALSE);
+	g_return_val_if_fail (GNOME_IS_CANVAS (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	retval = FALSE;
+
+	canvas = GNOME_CANVAS (widget);
+
+	d(g_print ("button %d, event type %d, grabbed=%p, current=%p\n",
+		   event->button,
+		   event->type,
+		   canvas->grabbed_item,
+		   canvas->current_item));
+
+        /* dispatch normally regardless of the event's window if an item has
+           has a pointer grab in effect */
+	if (!canvas->grabbed_item && event->window != canvas->layout.bin_window)
+		return retval;
+
+	switch (event->button) {
+	case 1:
+		mask = GDK_BUTTON1_MASK;
+		break;
+	case 2:
+		mask = GDK_BUTTON2_MASK;
+		break;
+	case 3:
+		mask = GDK_BUTTON3_MASK;
+		break;
+	case 4:
+		mask = GDK_BUTTON4_MASK;
+		break;
+	case 5:
+		mask = GDK_BUTTON5_MASK;
+		break;
+	default:
+		mask = 0;
+	}
+
+	switch (event->type) {
+	case GDK_BUTTON_PRESS:
+	case GDK_2BUTTON_PRESS:
+	case GDK_3BUTTON_PRESS:
+		/* Pick the current item as if the button were not pressed, and
+		 * then process the event.
+		 */
+		canvas->state = event->state;
+		pick_current_item (canvas, (GdkEvent *) event);
+		canvas->state ^= mask;
+		retval = emit_event (canvas, (GdkEvent *) event);
+		break;
+
+	case GDK_BUTTON_RELEASE:
+		/* Process the event as if the button were pressed, then repick
+		 * after the button has been released
+		 */
+		canvas->state = event->state;
+		retval = emit_event (canvas, (GdkEvent *) event);
+		event->state ^= mask;
+		canvas->state = event->state;
+		pick_current_item (canvas, (GdkEvent *) event);
+		event->state ^= mask;
+		break;
+
+	default:
+		g_assert_not_reached ();
+	}
+
+	return retval;
+}
+
 /* Key event handler for the canvas */
 static gint
 e_canvas_visibility (GtkWidget *widget, GdkEventVisibility *event, ECanvas *canvas)
@@ -348,7 +612,7 @@ e_canvas_item_grab_focus (GnomeCanvasItem *item, gboolean widget_too)
 		gtk_widget_grab_focus (GTK_WIDGET (item->canvas));
 	}
 
-	if (focused_item) {
+	if (item) {
 		ev.focus_change.type = GDK_FOCUS_CHANGE;
 		ev.focus_change.window = GTK_LAYOUT (item->canvas)->bin_window;
 		ev.focus_change.send_event = FALSE;
@@ -406,9 +670,36 @@ e_canvas_focus_out (GtkWidget *widget, GdkEventFocus *event)
 	}
 }
 
+static void
+ec_style_set_recursive (GnomeCanvasItem *item, GtkStyle *previous_style)
+{
+	guint signal_id = gtk_signal_lookup ("style_set", GTK_OBJECT_TYPE (item));
+	if (signal_id >= 1) {
+		GtkSignalQuery *query = gtk_signal_query (signal_id);
+		if (query->return_val == GTK_TYPE_NONE && query->nparams == 1 && query->params[0] == GTK_TYPE_STYLE) {
+			gtk_signal_emit (GTK_OBJECT (item), signal_id, previous_style);
+		}
+		g_free (query);
+	}
+
+	if (GNOME_IS_CANVAS_GROUP (item) ) {
+		GList *items = GNOME_CANVAS_GROUP (item)->item_list;
+		for (; items; items = items->next)
+			ec_style_set_recursive (items->data, previous_style);
+	}
+}
+
+static void
+e_canvas_style_set (GtkWidget *widget, GtkStyle *previous_style)
+{
+	ec_style_set_recursive (GNOME_CANVAS_ITEM (gnome_canvas_root (GNOME_CANVAS (widget))), previous_style);
+}
+
+
 static void
 e_canvas_realize (GtkWidget *widget)
 {
+	gint width, height;
 	ECanvas *ecanvas = E_CANVAS (widget);
 
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
@@ -423,12 +714,41 @@ e_canvas_realize (GtkWidget *widget)
 		GdkIMStyle style;
 		GdkIMStyle supported_style = GDK_IM_PREEDIT_NONE |
 			GDK_IM_PREEDIT_NOTHING |
+			GDK_IM_PREEDIT_POSITION |
 			GDK_IM_STATUS_NONE |
 			GDK_IM_STATUS_NOTHING;
 
+		if(widget->style && widget->style->font->type != GDK_FONT_FONTSET)
+			supported_style &= ~GDK_IM_PREEDIT_POSITION;
+
 		attr->style = style = gdk_im_decide_style (supported_style);
 		attr->client_window = ecanvas->parent.layout.bin_window;
 
+		switch (style & GDK_IM_PREEDIT_MASK)
+			{
+			case GDK_IM_PREEDIT_POSITION:
+				if (widget->style && widget->style->font->type != GDK_FONT_FONTSET)
+					{
+						g_warning ("over-the-spot style requires fontset");
+						break;
+					}
+				
+				gdk_window_get_size (attr->client_window, &width, &height);
+				height = widget->style->font->ascent +
+					widget->style->font->descent;
+
+				attrmask |= GDK_IC_PREEDIT_POSITION_REQ;
+				attr->spot_location.x = 0;
+				attr->spot_location.y = height;
+				attr->preedit_area.x = 0;
+				attr->preedit_area.y = 0;
+				attr->preedit_area.width = width;
+				attr->preedit_area.height = height;
+				attr->preedit_fontset = widget->style->font;
+
+				break;
+			}
+
 		ecanvas->ic = gdk_ic_new (attr, attrmask);
 		if (ecanvas->ic != NULL) {
 			mask = gdk_window_get_events (attr->client_window);
@@ -447,6 +767,11 @@ static void
 e_canvas_unrealize (GtkWidget *widget)
 {
 	ECanvas * ecanvas = E_CANVAS (widget);
+
+	if (ecanvas->idle_id)
+		g_source_remove(ecanvas->idle_id);
+	ecanvas->idle_id = 0;
+
 	if (ecanvas->ic) {
 		gdk_ic_destroy (ecanvas->ic);
 		ecanvas->ic = NULL;
@@ -477,8 +802,9 @@ e_canvas_item_invoke_reflow (GnomeCanvasItem *item, int flags)
 
 	if (item->object.flags & E_CANVAS_ITEM_NEEDS_REFLOW) {
 		ECanvasItemReflowFunc func;
-		func = gtk_object_get_data (GTK_OBJECT (item),
-					    "ECanvasItem::reflow_callback");
+		func = (ECanvasItemReflowFunc)
+			gtk_object_get_data (GTK_OBJECT (item),
+					     "ECanvasItem::reflow_callback");
 		if (func)
 			func (item, flags);
 	}
@@ -594,7 +920,7 @@ e_canvas_item_set_cursor (GnomeCanvasItem *item, gpointer id)
 	for (list = canvas->selection; list; list = g_list_next(list)) {
 		info = list->data;
 
-		func = gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_callback");
+		func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_callback");
 		if (func)
 			func(info->item, flags, info->id);
 		g_message ("ECANVAS: free info (2): item %p, id %p",
@@ -615,7 +941,7 @@ e_canvas_item_set_cursor (GnomeCanvasItem *item, gpointer id)
 	g_message ("ECANVAS: new info item %p, id %p", item, id);
 
 	flags = E_CANVAS_ITEM_SELECTION_SELECT | E_CANVAS_ITEM_SELECTION_CURSOR;
-	func = gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+	func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
 	if (func)
 		func(item, flags, id);
 
@@ -646,7 +972,7 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 	canvas = E_CANVAS(item->canvas);
 
 	if (canvas->cursor) {
-		func = gtk_object_get_data(GTK_OBJECT(canvas->cursor->item), "ECanvasItem::selection_callback");
+		func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(canvas->cursor->item), "ECanvasItem::selection_callback");
 		if (func)
 			func(canvas->cursor->item, flags, canvas->cursor->id);
 	}
@@ -661,11 +987,11 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 
 		if (search->item == item) {
 			ECanvasItemSelectionCompareFunc compare_func;
-			compare_func = gtk_object_get_data(GTK_OBJECT(search->item), "ECanvasItem::selection_compare_callback");
+			compare_func = (ECanvasItemSelectionCompareFunc)gtk_object_get_data(GTK_OBJECT(search->item), "ECanvasItem::selection_compare_callback");
 
 			if (compare_func(search->item, search->id, id, 0) == 0) {
 				canvas->cursor = search;
-				func = gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+				func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
 				if (func)
 					func(item, flags, search->id);
 				return;
@@ -679,7 +1005,7 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 	info->id = id;
 	g_message ("ECANVAS: new info (2): item %p, id %p", item, id);
 
-	func = gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+	func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
 	if (func)
 		func(item, flags, id);
 
@@ -708,11 +1034,11 @@ e_canvas_item_remove_selection (GnomeCanvasItem *item, gpointer id)
 
 		if (info->item == item) {
 			ECanvasItemSelectionCompareFunc compare_func;
-			compare_func = gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_compare_callback");
+			compare_func = (ECanvasItemSelectionCompareFunc)gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_compare_callback");
 
 			if (compare_func(info->item, info->id, id, 0) == 0) {
 				ECanvasItemSelectionFunc func;
-				func = gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_callback");
+				func = (ECanvasItemSelectionFunc) gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_callback");
 				if (func)
 					func(info->item, flags, info->id);
 				canvas->selection = g_list_remove_link(canvas->selection, list);
@@ -758,3 +1084,77 @@ void e_canvas_hide_tooltip  (ECanvas *canvas)
 		canvas->tooltip_window = NULL;
 	}
 }
+
+
+static gboolean
+grab_cancelled_check (gpointer data)
+{
+	ECanvas *canvas = data;
+
+	if (GNOME_CANVAS (canvas)->grabbed_item == NULL) {
+		canvas->grab_cancelled_cb = NULL;
+		canvas->grab_cancelled_check_id = 0;
+		canvas->grab_cancelled_time = 0;
+		canvas->grab_cancelled_data = NULL;
+		return FALSE;
+	}
+
+	if (gtk_grab_get_current ()) {
+		gnome_canvas_item_ungrab(GNOME_CANVAS (canvas)->grabbed_item, canvas->grab_cancelled_time);
+		if (canvas->grab_cancelled_cb) {
+			canvas->grab_cancelled_cb (canvas,
+						   GNOME_CANVAS (canvas)->grabbed_item,
+						   canvas->grab_cancelled_data);
+		}
+		canvas->grab_cancelled_cb = NULL;
+		canvas->grab_cancelled_check_id = 0;
+		canvas->grab_cancelled_time = 0;
+		canvas->grab_cancelled_data = NULL;
+		return FALSE;
+	}
+	return TRUE;
+}
+
+int
+e_canvas_item_grab (ECanvas *canvas,
+		    GnomeCanvasItem *item,
+		    guint event_mask,
+		    GdkCursor *cursor,
+		    guint32 etime,
+		    ECanvasItemGrabCancelled cancelled_cb,
+		    gpointer cancelled_data)
+{
+	if (gtk_grab_get_current ()) {
+		return AlreadyGrabbed;
+	} else {
+		int ret_val = gnome_canvas_item_grab (item, event_mask, cursor, etime);
+		if (ret_val == GrabSuccess) {
+			canvas->grab_cancelled_cb = cancelled_cb;
+			canvas->grab_cancelled_check_id =
+				g_timeout_add_full (G_PRIORITY_LOW,
+						    100,
+						    grab_cancelled_check,
+						    canvas,
+						    NULL);
+			canvas->grab_cancelled_time = etime;
+			canvas->grab_cancelled_data = cancelled_data;
+		}
+
+		return ret_val;
+	}
+}
+
+void
+e_canvas_item_ungrab (ECanvas *canvas,
+		      GnomeCanvasItem *item,
+		      guint32 etime)
+{
+	if (canvas->grab_cancelled_check_id) {
+		g_source_remove (canvas->grab_cancelled_check_id);
+		canvas->grab_cancelled_cb = NULL;
+		canvas->grab_cancelled_check_id = 0;
+		canvas->grab_cancelled_time = 0;
+		canvas->grab_cancelled_data = NULL;
+		gnome_canvas_item_ungrab (item, etime);
+	}
+}
