@@ -1,11 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-categories-master-list.c: the master list of categories.
+/* 
+ * e-categories-master-list-array.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
  * Authors:
  *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 1999, 2000 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 #include <config.h>
 
@@ -17,6 +30,7 @@
 #include <gnome-xml/parser.h>
 #include <gnome-xml/xmlmemory.h>
 #include "e-unicode.h"
+#include <stdlib.h>
 #include <string.h>
 
 #define PARENT_TYPE e_categories_master_list_get_type ()
@@ -25,78 +39,96 @@
 
 typedef enum {
 	E_CATEGORIES_MASTER_LIST_ARRAY_NORMAL,
-	E_CATEGORIES_MASTER_LIST_ARRAY_DELETED,
+	E_CATEGORIES_MASTER_LIST_ARRAY_DELETED
 } ECategoriesMasterListArrayState;
 
+typedef struct {
+	char *category;
+	char *icon;
+	char *color;
+} ECategoriesMasterListArrayItem;
+
 struct _ECategoriesMasterListArrayPriv {
-	char                            **categories;
+	ECategoriesMasterListArrayItem  **categories;
 	int                               count;
 
 	ECategoriesMasterListArrayState  *states;
-	char                            **appends;
+	ECategoriesMasterListArrayItem  **appends;
 	int                               appends_count;
-
-	GHashTable                       *icons;
-	GHashTable                       *colors;
 };
 
 static ECategoriesMasterListClass *parent_class;
 
-gchar *builtin_categories[] = {
-	N_("Birthday"),
-	N_("Business"),
-	N_("Competition"),
-	N_("Favorites"),
-	N_("Gifts"),
-	N_("Goals/Objectives"),
-	N_("Holiday"),
-	N_("Holiday Cards"),
-	N_("Hot Contacts"),
-	N_("Ideas"),
-	N_("International"),
-	N_("Key Customer"),
-	N_("Miscellaneous"),
-	N_("Personal"),
-	N_("Phone Calls"),
-	N_("Status"),
-	N_("Strategies"),
-	N_("Suppliers"),
-	N_("Time & Expenses"),
-	N_("VIP"),
-	N_("Waiting"),
+struct {
+	gchar *category;
+	gchar *icon;
+} builtin_categories[] = {
+	{ N_("Birthday"),         GAL_IMAGESDIR "/category_birthday_16.png" },
+	{ N_("Business"),         GAL_IMAGESDIR "/category_business_16.png" },
+	{ N_("Competition"),      NULL },
+	{ N_("Favorites"),        GAL_IMAGESDIR "/category_favorites_16.png" },
+	{ N_("Gifts"),            GAL_IMAGESDIR "/category_gifts_16.png" },
+	{ N_("Goals/Objectives"), GAL_IMAGESDIR "/category_goals_16.png" },
+	{ N_("Holiday"),          GAL_IMAGESDIR "/category_holiday_16.png" },
+	{ N_("Holiday Cards"),    GAL_IMAGESDIR "/category_holiday-cards_16.png" },
+	{ N_("Hot Contacts"),     GAL_IMAGESDIR "/category_hot-contacts_16.png" },
+	{ N_("Ideas"),            GAL_IMAGESDIR "/category_ideas_16.png" },
+	{ N_("International"),    GAL_IMAGESDIR "/category_international_16.png" },
+	{ N_("Key Customer"),     GAL_IMAGESDIR "/category_key-customer_16.png" },
+	{ N_("Miscellaneous"),    GAL_IMAGESDIR "/category_miscellaneous_16.png" },
+	{ N_("Personal"),         GAL_IMAGESDIR "/category_personal_16.png" },
+	{ N_("Phone Calls"),      GAL_IMAGESDIR "/category_phonecalls_16.png" },
+	{ N_("Status"),           GAL_IMAGESDIR "/category_status_16.png" },
+	{ N_("Strategies"),       GAL_IMAGESDIR "/category_strategies_16.png" },
+	{ N_("Suppliers"),        GAL_IMAGESDIR "/category_suppliers_16.png" },
+	{ N_("Time & Expenses"),  GAL_IMAGESDIR "/category_time-and-expenses_16.png" },
+	{ N_("VIP"),              NULL },
+	{ N_("Waiting"),          NULL },
 };
 
 #define BUILTIN_CATEGORY_COUNT (sizeof(builtin_categories) / sizeof(builtin_categories[0]))
 
-static void free_hash_string (gpointer key, gpointer value, gpointer user_data)
+static void
+ecmlai_free (ECategoriesMasterListArrayItem *ecmlai)
 {
-	g_free (value);
+	g_free (ecmlai->category);
+	g_free (ecmlai->icon);
+	g_free (ecmlai->color);
+	g_free (ecmlai);
+}
+
+static ECategoriesMasterListArrayItem *
+ecmlai_new (const char *category,
+	    const char *icon,
+	    const char *color)
+{
+	ECategoriesMasterListArrayItem *ret_val;
+	ret_val = g_new (ECategoriesMasterListArrayItem, 1);
+	ret_val->category = g_strdup (category);
+	ret_val->icon = g_strdup (icon);
+	ret_val->color = g_strdup (color);
+	return ret_val;
 }
 
 static void
-ecmla_clear (ECategoriesMasterListArray *ecmla)
+ecmla_free (ECategoriesMasterListArray *ecmla)
 {
 	int i;
+
 	g_free (ecmla->priv->states);
 	ecmla->priv->states = NULL;
 
 	for (i = 0; i < ecmla->priv->appends_count; i++)
-		g_free (ecmla->priv->appends[i]);
+		ecmlai_free (ecmla->priv->appends[i]);
 	g_free (ecmla->priv->appends);
 	ecmla->priv->appends = NULL;
 	ecmla->priv->appends_count = 0;
 
 	for (i = 0; i < ecmla->priv->count; i++)
-		g_free (ecmla->priv->categories[i]);
+		ecmlai_free (ecmla->priv->categories[i]);
 	g_free (ecmla->priv->categories);
 	ecmla->priv->categories = NULL;
 	ecmla->priv->count = 0;
-
-	g_hash_table_foreach (ecmla->priv->icons, (GHFunc) free_hash_string, NULL);
-	g_hash_table_foreach (ecmla->priv->colors, (GHFunc) free_hash_string, NULL);
-	g_hash_table_destroy (ecmla->priv->icons);
-	g_hash_table_destroy (ecmla->priv->colors);
-	ecmla->priv->icons = ecmla->priv->colors = NULL;
 }
 
 /**
@@ -128,7 +160,7 @@ ecmla_nth (ECategoriesMasterList *ecml, int n)
 	g_return_val_if_fail (n < ecmla->priv->count, NULL);
 	g_return_val_if_fail (n >= 0, NULL);
 
-	return ecmla->priv->categories[n];
+	return ecmla->priv->categories[n]->category;
 }
 
 /**
@@ -142,17 +174,12 @@ ecmla_nth (ECategoriesMasterList *ecml, int n)
 static const char *
 ecmla_nth_icon (ECategoriesMasterList *ecml, int n)
 {
-	char *category;
-	char *icon;
 	ECategoriesMasterListArray *ecmla = E_CATEGORIES_MASTER_LIST_ARRAY (ecml);
 
 	g_return_val_if_fail (n < ecmla->priv->count, NULL);
 	g_return_val_if_fail (n >= 0, NULL);
 
-	category = ecmla->priv->categories[n];
-	icon = g_hash_table_lookup (ecmla->priv->icons, category);
-
-	return (const char *) icon;
+	return ecmla->priv->categories[n]->icon;
 }
 
 /**
@@ -166,17 +193,12 @@ ecmla_nth_icon (ECategoriesMasterList *ecml, int n)
 static const char *
 ecmla_nth_color (ECategoriesMasterList *ecml, int n)
 {
-	char *category;
-	char *color;
 	ECategoriesMasterListArray *ecmla = E_CATEGORIES_MASTER_LIST_ARRAY (ecml);
 
 	g_return_val_if_fail (n < ecmla->priv->count, NULL);
 	g_return_val_if_fail (n >= 0, NULL);
 
-	category = ecmla->priv->categories[n];
-	color = g_hash_table_lookup (ecmla->priv->colors, category);
-
-	return (const char *) color;
+	return ecmla->priv->categories[n]->color;
 }
 
 /**
@@ -192,31 +214,11 @@ ecmla_append (ECategoriesMasterList *ecml,
 	      const char *color,
 	      const char *icon)
 {
-	char *str;
 	ECategoriesMasterListArray *ecmla = E_CATEGORIES_MASTER_LIST_ARRAY (ecml);
 
-	ecmla->priv->appends = g_renew(char *, ecmla->priv->appends, ecmla->priv->appends_count + 1);
-	ecmla->priv->appends[ecmla->priv->appends_count] = g_strdup (category);
+	ecmla->priv->appends = g_renew(ECategoriesMasterListArrayItem *, ecmla->priv->appends, ecmla->priv->appends_count + 1);
+	ecmla->priv->appends[ecmla->priv->appends_count] = ecmlai_new (category, icon, color);
 	ecmla->priv->appends_count ++;
-
-	/* add icon/color */
-	str = g_hash_table_lookup (ecmla->priv->icons, category);
-	if (str != NULL) {
-		g_hash_table_remove (ecmla->priv->icons, category);
-		g_free (str);
-	}
-	g_hash_table_insert (ecmla->priv->icons,
-			     (gpointer) category,
-			     (gpointer) g_strdup (icon));
-
-	str = g_hash_table_lookup (ecmla->priv->colors, category);
-	if (str != NULL) {
-		g_hash_table_remove (ecmla->priv->colors, category);
-		g_free (str);
-	}
-	g_hash_table_insert (ecmla->priv->colors,
-			     (gpointer) category,
-			     (gpointer) g_strdup (color));
 }
 
 /**
@@ -252,7 +254,7 @@ ecmla_commit (ECategoriesMasterList *ecml)
 	int count;
 	int i;
 	int j;
-	char **new_list;
+	ECategoriesMasterListArrayItem **new_list;
 
 	if (ecmla->priv->states != NULL) {
 		count = 0;
@@ -266,14 +268,14 @@ ecmla_commit (ECategoriesMasterList *ecml)
 
 	count += ecmla->priv->appends_count;
 
-	new_list = g_new (char *, count);
+	new_list = g_new (ECategoriesMasterListArrayItem *, count);
 
 	j = 0;
 	for (i = 0; i < ecmla->priv->count; i++) {
 		if (ecmla->priv->states == NULL || ecmla->priv->states[i] == E_CATEGORIES_MASTER_LIST_ARRAY_NORMAL)
 			new_list[j++] = ecmla->priv->categories[i];
 		else
-			g_free (ecmla->priv->categories[i]);
+			ecmlai_free (ecmla->priv->categories[i]);
 	}
 	for (i = 0; i < ecmla->priv->appends_count; i++ )
 		new_list[j++] = ecmla->priv->appends[i];
@@ -291,6 +293,20 @@ ecmla_commit (ECategoriesMasterList *ecml)
 	e_categories_master_list_changed (ecml);
 }
 
+static void
+ecmla_default (ECategoriesMasterListArray *ecmla)
+{
+	int i;
+
+	ecmla->priv->count = BUILTIN_CATEGORY_COUNT;
+	ecmla->priv->categories = g_new (ECategoriesMasterListArrayItem *, BUILTIN_CATEGORY_COUNT);
+
+	for (i = 0; i < BUILTIN_CATEGORY_COUNT; i++) {
+		char *category = e_utf8_from_locale_string(_(builtin_categories[i].category));
+		ecmla->priv->categories[i] = ecmlai_new (category, builtin_categories[i].icon, NULL);
+		g_free (category);
+	}
+}
 
 /**
  * ecmla_reset:
@@ -300,16 +316,9 @@ static void
 ecmla_reset (ECategoriesMasterList *ecml)
 {
 	ECategoriesMasterListArray *ecmla = E_CATEGORIES_MASTER_LIST_ARRAY (ecml);
-	int i;
 
-	ecmla_clear (ecmla);
-
-	ecmla->priv->count = BUILTIN_CATEGORY_COUNT;
-	ecmla->priv->categories = g_new (char *, BUILTIN_CATEGORY_COUNT);
-
-	for (i = 0; i < BUILTIN_CATEGORY_COUNT; i++) {
-		ecmla->priv->categories[i] = e_utf8_from_locale_string(_(builtin_categories[i]));
-	}
+	ecmla_free (ecmla);
+	ecmla_default (ecmla);
 }
 
 static void
@@ -317,9 +326,12 @@ ecmla_destroy (GtkObject *object)
 {
 	ECategoriesMasterListArray *ecmla = E_CATEGORIES_MASTER_LIST_ARRAY (object);
 
-	ecmla_clear (ecmla);
+	ecmla_free (ecmla);
 	g_free (ecmla->priv);
 	ecmla->priv = NULL;
+
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 
@@ -346,21 +358,14 @@ ecmla_class_init (GtkObjectClass *object_class)
 static void
 ecmla_init (ECategoriesMasterListArray *ecmla)
 {
-	int i;
-
-	ecmla->priv = g_new (ECategoriesMasterListArrayPriv, 1);
-	ecmla->priv->count = BUILTIN_CATEGORY_COUNT;
-	ecmla->priv->categories = g_new (char *, BUILTIN_CATEGORY_COUNT);
-	ecmla->priv->states = NULL;
-	ecmla->priv->appends = NULL;
+	ecmla->priv                = g_new (ECategoriesMasterListArrayPriv, 1);
+	ecmla->priv->count         = 0;
+	ecmla->priv->categories    = NULL;
+	ecmla->priv->states        = NULL;
+	ecmla->priv->appends       = NULL;
 	ecmla->priv->appends_count = 0;
 
-	ecmla->priv->icons = g_hash_table_new (g_str_hash, g_str_equal);
-	ecmla->priv->colors = g_hash_table_new (g_str_hash, g_str_equal);
-
-	for (i = 0; i < BUILTIN_CATEGORY_COUNT; i++) {
-		ecmla->priv->categories[i] = e_utf8_from_locale_string(_(builtin_categories[i]));
-	}
+	ecmla_default (ecmla);
 }
 
 guint
@@ -401,7 +406,7 @@ e_categories_master_list_array_from_string (ECategoriesMasterListArray *ecmla,
 	xmlNode *children;
 	int count;
 	int i;
-	char **categories;
+	ECategoriesMasterListArrayItem **categories;
 
 	char *string_copy = g_strdup (string);
 	doc = xmlParseMemory(string_copy, strlen (string_copy));
@@ -416,17 +421,18 @@ e_categories_master_list_array_from_string (ECategoriesMasterListArray *ecmla,
 		count ++;
 	}
 
-	categories = g_new (char *, count);
+	categories = g_new (ECategoriesMasterListArrayItem *, count);
 	i = 0;
 
 	for (children = node->xmlChildrenNode;
 	     children;
 	     children = children->next) {
-		categories[i++] = e_xml_get_string_prop_by_name (children,
-								 "a");
+		categories[i++] = ecmlai_new (e_xml_get_string_prop_by_name (children, "a"),
+					      e_xml_get_string_prop_by_name (children, "icon"),
+					      e_xml_get_string_prop_by_name (children, "color"));
 	}
 
-	ecmla_clear (ecmla);
+	ecmla_free (ecmla);
 	ecmla->priv->count = count;
 	ecmla->priv->categories = categories;
 
@@ -452,7 +458,11 @@ e_categories_master_list_array_to_string (ECategoriesMasterListArray *ecmla)
 
 	for (i = 0; i < ecmla->priv->count; i++) {
 		child = xmlNewChild (node, NULL, "cat", NULL);
-		e_xml_set_string_prop_by_name (child, "a", ecmla->priv->categories[i]);
+		e_xml_set_string_prop_by_name (child, "a", ecmla->priv->categories[i]->category);
+		if (ecmla->priv->categories[i]->color)
+			e_xml_set_string_prop_by_name (child, "color", ecmla->priv->categories[i]->color);
+		if (ecmla->priv->categories[i]->icon)
+			e_xml_set_string_prop_by_name (child, "icon", ecmla->priv->categories[i]->icon);
 	}
 	xmlDocDumpMemory (doc, &temp, &length);
 	string = g_strdup (temp);
