@@ -1,13 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * GUI utility functions
+/* 
+ * e-gui-utils.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza <miguel@helixcode.com>
+ * Authors:
+ *   Miguel de Icaza <miguel@ximian.com>
  *
- * Copyright (C) 1999 Miguel de Icaza
- * Copyright (C) 2000, 2001 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 
 #include "e-gui-utils.h"
@@ -125,6 +138,9 @@ e_container_change_tab_order_callback(GtkContainer *container,
 
 	focus_child = container->focus_child;
 
+	if (focus_child == NULL)
+		return FALSE;
+
 	if (direction == GTK_DIR_TAB_BACKWARD) {
 		children = g_list_last(children);
 	}
@@ -213,3 +229,34 @@ e_container_focus_nth_entry(GtkContainer *container, int n)
 	if (data.widget)
 		gtk_widget_grab_focus(data.widget);
 }
+
+gboolean
+e_glade_xml_connect_widget (GladeXML *gui, char *name, char *signal, GtkSignalFunc cb, gpointer closure)
+{
+	GtkWidget *widget;
+
+	widget = glade_xml_get_widget (gui, name);
+
+	if (widget) {
+		gtk_signal_connect (GTK_OBJECT (widget), signal,
+				    cb, closure);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+e_glade_xml_set_sensitive (GladeXML *gui, char *name, gboolean sensitive)
+{
+	GtkWidget *widget;
+
+	widget = glade_xml_get_widget (gui, name);
+
+	if (widget) {
+		gtk_widget_set_sensitive (widget, sensitive);
+		return TRUE;
+	}
+
+	return FALSE;
+}
